#include "StdAfx.h"
#include ".\bmssequence.hpp"

namespace Lmt {
namespace Max2Bms {
	BmsSequence::BmsSequence( void ) {
		this->Clear();
	}

	BmsSequence::~BmsSequence( void ) {
		m_measures.clear();
	}

	int BmsSequence::GetLastMeasureNo( void ) const {
		return m_lastMeasureNo;
	}

	void BmsSequence::SetBackgroundIndex( int index ) {
		if ((index < 0) || (index >= MaxBackgroundCount)) {
			return;
		}
		m_bkIndex = index;
	}

	const BmsMeasure * BmsSequence::GetMeasure( BmsChannels ch, int measure ) const {
		int index;
		BmsMeasureMap::const_iterator it;

		index = BmsSequence::ChannelToIndex( ch );
		if (index == -1) {
			return NULL;
		}

		it = m_measures[index].find( measure );
		if (it == m_measures[index].end()) {
			return NULL;
		}

		return &it->second;
	}

	void BmsSequence::Clear( void ) {
		m_measures.clear();

		m_measures.resize( BackgroundOffset + MaxBackgroundCount );
		m_lastMeasureNo = 0;
		m_bkIndex = 0;
	}

	void BmsSequence::Add( BmsChannels ch, int position, AlphaDecimal value ) {
		int indexCh;
		int measure;
		int indexMeasure;
		BmsMeasureMap::iterator it;

		indexCh = BmsSequence::ChannelToIndex( ch );
		if (indexCh == -1) {
			return;
		}

		if (position < 0) {
			return;
		}
		BmsSequence::PositionToMeasure( position, &measure, &indexMeasure );

		it = m_measures[indexCh].find( measure );
		if (it == m_measures[indexCh].end()) {
			it = m_measures[indexCh].insert( make_pair( measure, BmsMeasure() ) ).first;
		}

		it->second[indexMeasure] = value;

		if (measure > m_lastMeasureNo) {
			m_lastMeasureNo = measure;
		}
	}

	int BmsSequence::ChannelToIndex( BmsChannels ch ) const {
		switch (ch) {
			case BackgroundBmsCh:
				return BackgroundOffset + m_bkIndex;
			case P1Line1BmsCh:
				return PlayableKey1;
			case P1Line2BmsCh:
				return PlayableKey2;
			case P1Line3BmsCh:
				return PlayableKey3;
			case P1Line4BmsCh:
				return PlayableKey4;
			case P1Line5BmsCh:
				return PlayableKey5;
			case P1Line6BmsCh:
				return PlayableKey6;
			case P1Line7BmsCh:
				return PlayableKey7;
			case P1Ln1BmsCh:
				return LongNote1;
			case P1Ln2BmsCh:
				return LongNote2;
			case P1Ln3BmsCh:
				return LongNote3;
			case P1Ln4BmsCh:
				return LongNote4;
			case P1Ln5BmsCh:
				return LongNote5;
			case P1Ln6BmsCh:
				return LongNote6;
			case P1Ln7BmsCh:
				return LongNote7;
			case ChangeTempoBmsCh:
				return ChangeTempo;
			default:
				return -1;
		}
	}

	void BmsSequence::PositionToMeasure( int position, int *pmeasure, int *pindex ) const {
		*pmeasure = (position / BmsMeasure::Resolution) + 1;
		*pindex = position % BmsMeasure::Resolution;
	}
}
}
