#include "StdAfx.h"
#include "Track.hpp"
#include "Utility.hpp"

namespace Lmt {
namespace Max2Bms {
	Track::Track( void ) {
		m_ptrackRaw = NULL;
		m_pheader = NULL;
		m_pelements = NULL;
	}

	Track::~Track(void) {
	}

	int Track::GetElementCount( void ) const {
		return m_pheader->dataLength / sizeof(PtFormat::TrackObject);
	}

	wstring Track::GetTrackName( void ) const {
		return Utility::StringToWString( m_pheader->trackName );
	}

	bool Track::GetElement( int index, TrackElement &element ) const {
		int ecount = this->GetElementCount();

		if ((index < 0) || (index >= ecount)) {
			return false;
		}

		if (element.Initialize( m_pelements + (index * sizeof(PtFormat::TrackObject)) ) == false) {
			return false;
		}

		return true;
	}

	bool Track::Initialize( const unsigned char *ptrackRaw ) {
		const PtFormat::TrackHeader *pheader = reinterpret_cast<const PtFormat::TrackHeader*>( ptrackRaw );
		const unsigned char *pelements = ptrackRaw + sizeof(PtFormat::TrackHeader);

		if (memcmp( pheader->signature, "EZTR", 4 ) != 0) {
			return false;
		}
		if ((pheader->dataLength % sizeof(PtFormat::TrackObject)) != 0) {
			return false;
		}

		m_ptrackRaw = ptrackRaw;
		m_pheader = pheader;
		m_pelements = pelements;

		return true;
	}
}
}
